####################################
# default configuration and traces #
####################################

proc default_config {} {
  # set up default config
  lappend conf Audio,Samplerate 48000
  lappend conf Audio,Input,Device {audio:-1}
  lappend conf Audio,Output,Device {audio:-1}
  lappend conf Audio,Bitdepth 16
  # expected soundcard latency in ms
  # used for computing the total record length
  lappend conf Audio,Latency 200

  # The output channel and level definition
  lappend conf Output,Channel Left
  # output RMS level
  # The output level wrt full scale is computed by:
  # lvl = Output,Level / (Output,{Global|Left|Right},Gain * Output,{Global|Left|Right},Sensitivity)
  # Actually, the program assumes a test signal with 3 dB crest factor. As this is mostly true for sweeps, pseudo-random noises have usually one of 10 to 12 dB.
  lappend conf Output,Level 2.83
  # maximum peak FS level to prevent nasty clipping
  lappend conf Output,MaxFS 0.99
  # gain of external amplifier
  lappend conf Output,Global,Gain 1.0
  # sensitivity of soundcard output in Vrms/FS
  lappend conf Output,Global,Sensitivity 1.0
  lappend conf Output,Left,Gain 1.0
  lappend conf Output,Left,Sensitivity 1.0
  lappend conf Output,Right,Gain 1.0
  lappend conf Output,Right,Sensitivity 1.0

  # DUT channel for single and dual channel measurements
  lappend conf Input,Channel Left
  # Reference channel for dual channel measurements
  lappend conf Input,Reference Right
  # Gain of external amplifier
  lappend conf Input,Global,Gain 1.0
  # Sensitivity of soundcard input in Vrms/FS
  lappend conf Input,Global,Sensitivity 1.0
  lappend conf Input,Left,Gain 1.0
  lappend conf Input,Left,Sensitivity 1.0
  lappend conf Input,Right,Gain 1.0
  lappend conf Input,Right,Sensitivity 1.0

  # currently loaded signal definition
  lappend conf Signal,Definition,Bandwidth {{20 20e3}}
  lappend conf Signal,Definition,Bins 60
  lappend conf Signal,Definition,Spectrum {pink}
  lappend conf Signal,Definition,Phase {random}
  lappend conf Signal,Definition,Spacing {log}
  # minimall signal size. Should be a power of two for efficient FFT calculation
  lappend conf Signal,MinSize 48000

  # The bandwidth used during evaluation
  lappend conf Processing,Bandwidth {{20 20e3}}
  lappend conf Processing,Bins 200
  lappend conf Processing,Spacing {log}
   
  # display scaling:
  # auto: full automatic scaling
  # manual: manual scaling
  lappend conf Display,Scaling {auto}
  # display range when doing manual scaling
  lappend conf Display,Range {{0 100}}
  # bound for automatic scaling; if empty, no bounds are applied
  lappend conf Display,Bounds {}

  # Physical constants
  lappend conf Physics,C 343.0

  lappend conf System,LastDir {}

  # create ensemble command
  ::config::makeEnsemble ::setup
  # write defaults to configuration array
  setup aset $conf

  # read setup from file
  if {![::config::readSetup]} {
    ::config::writeSetup
  }

  # install some traces
  setup ctrace add Audio,Samplerate write [list apply {{v1 v2 op} {
      upvar $v1 options
      audio configure Samplerate $options($v2)
      catch {updateIMP}
    }}]
  setup ctrace add Audio,Input,Device write [list apply {{v1 v2 op} {
      upvar $v1 options
      audio configure Input,Device $options($v2)
    }}]
  setup ctrace add Audio,Output,Device write [list apply {{v1 v2 op} {
      upvar $v1 options
      audio configure Output,Device $options($v2)
    }}]
  setup ctrace add Display,* write [list apply {{v1 v2 op} {
      catch {updateIMP}
      catch {updateAppTitle IMP}
    }}]
}

proc load_config {} {
  ::config::readSetup
  # enable save of setup on every change
  #::config::enable_writeOnChange
}

