############
# File I/O #
############

proc :w {{fname ""} {overwrite no} {create no}} {
  if {[setup cget System,LastDir] eq {}} {
    set initdir [file join [pwd]]
  } else {
    set initdir [setup cget System,LastDir]
  }
  # get all overlays
  if {[llength [::overlay::ids]] <=0} {
    return -code break {Nothing to save} -bg yellow
  }
  if {$fname == {}} {
    if {[set fname [tk_getSaveFile \
      -initialdir $initdir \
      -filetypes { \
        {{IMD files} {.imd}} \
        {{Text files} {.txt}} \
        {{All files} *} \
        } \
      -defaultextension {.imd}  \
      ]] == {}} {
      return -code break {Save cancelled} -bg yellow
    }
  }
  setup configure System,LastDir [file dirname $fname]
  
  if {[set fp [open $fname w]] eq {}} {
    return -code error "Couldn't open file $fname"
  }
  # write each IR to a file and fill index file
  foreach {ID propList} [::overlay::get all] {
    puts $fp "$ID {"
    puts $fp $propList
    puts $fp "}"
  }
  close $fp

  # success
  return -code ok "Saved to $fname"
}

proc :l {{fname {}} {replace no} {ignore no}} {
  if {[setup cget System,LastDir] eq {}} {
    set initdir [file join [pwd]]
  } else {
    set initdir [setup cget System,LastDir]
  }
  if {$fname == {}} {
    if {[set fname [tk_getOpenFile \
      -initialdir $initdir \
      -filetypes { \
        {{IMD files} {.imd}} \
        {{Text files} {.txt}} \
        {{All files} *} \
        } \
      -defaultextension {.imd}  \
      ]] == {}} {
      return -code break {Load cancelled}
    }
  }
  setup configure System,LastDir [file dirname $fname]
  if {[set fp [open $fname r]] eq {}} {
    return -code error "Couldn't open file $fname"
  }

  # read file at once
  set def [read $fp]
  close $fp
  
  # remove comments
  set def [regsub -all {^#.*} $def { }]
  # remove newlines
  set def [regsub -all {\n} $def { }]
  # remove "bad" symbols
  set def [regsub -all {[\[\]]} $def { }]

  # delete old overlays if wanted
  if {$replace} {
    ::overlay::delete all
    ::TkEsweepXYPlot::removeTrace IMD all
  }
  # parse
  foreach {ID propList} $def {
    ::overlay::add $propList
  }

  # success
  updateIMD
  return -code ok "$fname loaded" 
}

